package ovh.herisson.Clyde.EndPoints;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.CourseService;
import ovh.herisson.Clyde.Services.TeacherCourseService;
import ovh.herisson.Clyde.Tables.Course;
import ovh.herisson.Clyde.Tables.Role;
import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
public class CourseController {

    private final CourseService courseServ;

    private final TeacherCourseService teacherCourseServ;

    private final AuthenticatorService authServ;

    public CourseController(CourseService courseServ, TeacherCourseService teacherCourseServ, AuthenticatorService authServ) {
        this.courseServ = courseServ;
        this.teacherCourseServ = teacherCourseServ;
        this.authServ = authServ;
    }

    @GetMapping("/course/{id}")
    public ResponseEntity<Course> getCourse(@RequestHeader("Authorization") String token, @PathVariable long id){
        if (authServ.getUserFromToken(token) == null)
            return new UnauthorizedResponse<>(null);

        Course foundCourse = courseServ.findById(id);

        if (foundCourse == null)
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(foundCourse, HttpStatus.OK);
    }


    @PostMapping("/course")
    public ResponseEntity<Course> postCourse(@RequestHeader("Authorization") String token,
                                             @RequestBody Course course)
    {

        if (authServ.IsNotIn(new Role[]{Role.Secretary,Role.Admin},token))
            return new UnauthorizedResponse<>(null);

        return new ResponseEntity<>(courseServ.save(course), HttpStatus.CREATED);
    }


    @PatchMapping("/course/{id}")
    public ResponseEntity<Course> patchCourse(@RequestHeader("Authorization") String token,
                                              @RequestBody Map<String,Object> updates,
                                              @PathVariable long id)
    {

        if (authServ.IsNotIn(new Role[]{Role.Admin,Role.Teacher,Role.Secretary}, token))
            return new UnauthorizedResponse<>(null);

        Course modifiedCourse = courseServ.modifyData(id,updates,authServ.getUserFromToken(token).getRole());

        if (modifiedCourse == null)
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(modifiedCourse, HttpStatus.OK);
    }

    @PostMapping("/course/{id}")
    public ResponseEntity<String> postTeachers(@RequestHeader("Authorization") String token,
                                               @RequestBody Iterable<Long> teacherIds,
                                               @PathVariable Long id)
    {

        if (authServ.IsNotIn(new Role[]{Role.Admin,Role.Secretary}, token))
            return new UnauthorizedResponse<>(null);


        if (!teacherCourseServ.saveAll(teacherIds,courseServ.findById(id)))
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(HttpStatus.OK);
    }
}
